@@ -33,14 +33,28 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
-#define NM_VPN_API_SUBJECT_TO_CHANGE
-
+#ifdef NM_IODINE_OLD
+#define NM_VPN_LIBNM_COMPAT
 #include <nm-vpn-plugin-ui-interface.h>
 #include <nm-setting-vpn.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
 
-#include "../src/nm-iodine-service.h"
+#define nm_simple_connection_new nm_connection_new
+
+#define IODINE_EDITOR_PLUGIN_ERROR                     NM_SETTING_VPN_ERROR
+#define IODINE_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY    NM_SETTING_VPN_ERROR_INVALID_PROPERTY
+ 
+#else /* !NM_IODINE_OLD */
+
+#include <NetworkManager.h>
+#include <nma-ui-utils.h>
+
+#define IODINE_EDITOR_PLUGIN_ERROR                     NM_CONNECTION_ERROR
+#define IODINE_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY    NM_CONNECTION_ERROR_INVALID_PROPERTY
+#endif
+
+#include "nm-iodine-service-defines.h"
 #include "nm-iodine.h"
 
 #define IODINE_PLUGIN_NAME    _("Iodine DNS Tunnel")
@@ -53,37 +67,60 @@
 
 /************** plugin class **************/
 
-static void iodine_plugin_ui_interface_init (NMVpnPluginUiInterface
-											 *iface_class);
+enum {
+	PROP_0,
+	PROP_NAME,
+	PROP_DESC,
+	PROP_SERVICE
+};
 
-G_DEFINE_TYPE_EXTENDED (IodinePluginUi, iodine_plugin_ui, G_TYPE_OBJECT, 0,
-						G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_PLUGIN_UI_INTERFACE,
-											   iodine_plugin_ui_interface_init))
+static void iodine_editor_plugin_interface_init (NMVpnEditorPluginInterface *iface_class);
+
+G_DEFINE_TYPE_EXTENDED (IodineEditorPlugin, iodine_editor_plugin, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_EDITOR_PLUGIN,
+                                               iodine_editor_plugin_interface_init))
 
 /************** UI widget class **************/
 
-static void iodine_plugin_ui_widget_interface_init (NMVpnPluginUiWidgetInterface
-													*iface_class);
+static void iodine_editor_interface_init (NMVpnEditorInterface *iface_class);
 
-G_DEFINE_TYPE_EXTENDED (IodinePluginUiWidget, iodine_plugin_ui_widget,G_TYPE_OBJECT, 0,
-						G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_PLUGIN_UI_WIDGET_INTERFACE,
-											   iodine_plugin_ui_widget_interface_init))
+G_DEFINE_TYPE_EXTENDED (IodineEditor, iodine_editor,G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_EDITOR,
+                                               iodine_editor_interface_init))
 
-#define IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IODINE_TYPE_PLUGIN_UI_WIDGET, IodinePluginUiWidgetPrivate))
+#define IODINE_EDITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IODINE_TYPE_EDITOR, IodineEditorPrivate))
 
 typedef struct {
 	GtkBuilder *builder;
 	GtkWidget *widget;
 	GtkSizeGroup *group;
 	gboolean window_added;
-} IodinePluginUiWidgetPrivate;
+} IodineEditorPrivate;
+
+typedef enum {
+	NM_IODINE_IMPORT_EXPORT_ERROR_UNKNOWN = 0,
+	NM_IODINE_IMPORT_EXPORT_ERROR_NOT_IODINE,
+	NM_IODINE_IMPORT_EXPORT_ERROR_BAD_DATA,
+} NMIodineImportError;
+
+#define NM_IODINE_IMPORT_EXPORT_ERROR nm_iodine_import_export_error_quark ()
+
+static GQuark
+nm_iodine_import_export_error_quark (void)
+{
+	static GQuark quark = 0;
+
+	if (G_UNLIKELY (quark == 0))
+		quark = g_quark_from_static_string ("nm-iodine-import-export-error-quark");
+	return quark;
+}
 
 static NMConnection *
-import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
+import (NMVpnEditorPlugin *iface, const char *path, GError **error)
 {
 	NMConnection *connection;
 	NMSettingConnection *s_con;
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	NMSettingIP4Config *s_ip4;
 	GKeyFile *keyfile;
 	GKeyFileFlags flags;
@@ -94,22 +131,22 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 
 	if (!g_key_file_load_from_file (keyfile, path, flags, error)) {
 		g_set_error (error,
-					 0,
-					 0,
-					 "does not look like a %s VPN connection (parse failed)",
-					 IODINE_PLUGIN_NAME);
+		             NM_IODINE_IMPORT_EXPORT_ERROR,
+		             NM_IODINE_IMPORT_EXPORT_ERROR_NOT_IODINE,
+		             "does not look like a %s VPN connection (parse failed)",
+		             IODINE_PLUGIN_NAME);
 		return NULL;
 	}
 
-	connection = nm_connection_new ();
+	connection = nm_simple_connection_new ();
 	s_con = NM_SETTING_CONNECTION (nm_setting_connection_new ());
 	nm_connection_add_setting (connection, NM_SETTING (s_con));
 
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
 	g_object_set (s_vpn,
-				  NM_SETTING_VPN_SERVICE_TYPE,
-				  NM_DBUS_SERVICE_IODINE,
-				  NULL);
+	              NM_SETTING_VPN_SERVICE_TYPE,
+	              NM_DBUS_SERVICE_IODINE,
+	              NULL);
 	nm_connection_add_setting (connection, NM_SETTING (s_vpn));
 
 	s_ip4 = NM_SETTING_IP4_CONFIG (nm_setting_ip4_config_new ());
@@ -121,11 +158,11 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 		nm_setting_vpn_add_data_item (s_vpn, NM_IODINE_KEY_TOPDOMAIN, buf);
 	} else {
 		g_set_error (error,
-					 0,
-					 0,
-					 "does not look like a %s VPN connection "
-					 "(no top level domain)",
-					 IODINE_PLUGIN_NAME);
+		             NM_IODINE_IMPORT_EXPORT_ERROR,
+		             NM_IODINE_IMPORT_EXPORT_ERROR_NOT_IODINE,
+		             "does not look like a %s VPN connection "
+		             "(no top level domain)",
+		             IODINE_PLUGIN_NAME);
 		g_object_unref (connection);
 		return NULL;
 	}
@@ -150,13 +187,12 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 }
 
 static gboolean
-export (NMVpnPluginUiInterface *iface,
+export (NMVpnEditorPlugin *iface,
         const char *path,
         NMConnection *connection,
         GError **error)
 {
-	NMSettingConnection *s_con;
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	const char *value;
 	const char *topdomain = NULL;
 	const char *nameserver = NULL;
@@ -166,22 +202,23 @@ export (NMVpnPluginUiInterface *iface,
 
 	f = fopen (path, "w");
 	if (!f) {
-		g_set_error (error, 0, 0, "could not open file for writing");
+		g_set_error (error,
+		             NM_IODINE_IMPORT_EXPORT_ERROR,
+		             NM_IODINE_IMPORT_EXPORT_ERROR_UNKNOWN,
+		             "could not open file for writing");
 		return FALSE;
 	}
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting
-								   (connection, NM_TYPE_SETTING_CONNECTION));
-
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection,
-														NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_IODINE_KEY_TOPDOMAIN);
 	if (value && strlen (value))
 		topdomain = value;
 	else {
-		g_set_error (error, 0, 0,
-					 "connection was incomplete (missing top level domain)");
+		g_set_error (error,
+		             NM_IODINE_IMPORT_EXPORT_ERROR,
+		             NM_IODINE_IMPORT_EXPORT_ERROR_UNKNOWN,
+		             "connection was incomplete (missing top level domain)");
 		goto done;
 	}
 
@@ -194,15 +231,15 @@ export (NMVpnPluginUiInterface *iface,
 		fragsize = value;
 
 	fprintf (f,
-			 "[iodine]\n"
-			 "Description=%s\n"
-			 "Topdomain=%s\n"
-			 "Nameserver=%s\n"
-			 "Fragsize=%s\n",
-			 /* Description */ nm_setting_connection_get_id (s_con),
-			 /* Topdomain */   topdomain,
-			 /* Nameserver */  nameserver,
-			 /* Fragsize */    fragsize);
+	         "[iodine]\n"
+	         "Description=%s\n"
+	         "Topdomain=%s\n"
+	         "Nameserver=%s\n"
+	         "Fragsize=%s\n",
+	         /* Description */ nm_connection_get_id (connection),
+	         /* Topdomain */   topdomain,
+	         /* Nameserver */  nameserver,
+	         /* Fragsize */    fragsize);
 
 	success = TRUE;
 
@@ -211,58 +248,20 @@ done:
 	return success;
 }
 
-GQuark
-iodine_plugin_ui_error_quark (void)
-{
-	static GQuark error_quark = 0;
-
-	if (G_UNLIKELY (error_quark == 0))
-		error_quark = g_quark_from_static_string ("iodine-plugin-ui-error-quark");
-
-	return error_quark;
-}
-
-/* This should really be standard. */
-#define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
-
-GType
-iodine_plugin_ui_error_get_type (void)
-{
-	static GType etype = 0;
-
-	if (etype == 0) {
-		static const GEnumValue values[] = {
-			/* Unknown error. */
-			ENUM_ENTRY (IODINE_PLUGIN_UI_ERROR_UNKNOWN,
-						"UnknownError"),
-			/* The specified property was invalid. */
-			ENUM_ENTRY (IODINE_PLUGIN_UI_ERROR_INVALID_PROPERTY,
-						"InvalidProperty"),
-			/* The specified property was missing and is required. */
-			ENUM_ENTRY (IODINE_PLUGIN_UI_ERROR_MISSING_PROPERTY,
-						"MissingProperty"),
-			{ 0, 0, 0 }
-		};
-		etype = g_enum_register_static ("IodinePluginUiError", values);
-	}
-	return etype;
-}
-
 static gboolean
-check_validity (IodinePluginUiWidget *self, GError **error)
+check_validity (IodineEditor *self, GError **error)
 {
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	IodineEditorPrivate *priv = \
+		IODINE_EDITOR_GET_PRIVATE (self);
 	GtkWidget *widget;
 	const char *str;
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "topdomain_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "topdomain_entry"));
 	str = gtk_entry_get_text (GTK_ENTRY (widget));
 	if (!str || !strlen (str)) {
 		g_set_error (error,
-		             IODINE_PLUGIN_UI_ERROR,
-		             IODINE_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             IODINE_EDITOR_PLUGIN_ERROR,
+		             IODINE_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_IODINE_KEY_TOPDOMAIN);
 		return FALSE;
 	}
@@ -273,17 +272,17 @@ check_validity (IodinePluginUiWidget *self, GError **error)
 static void
 stuff_changed_cb (GtkWidget *widget, gpointer user_data)
 {
-	g_signal_emit_by_name (IODINE_PLUGIN_UI_WIDGET (user_data), "changed");
+	g_signal_emit_by_name (IODINE_EDITOR (user_data), "changed");
 }
 
 static void
-setup_password_widget (IodinePluginUiWidget *self,
+setup_password_widget (IodineEditor *self,
                        const char *entry_name,
-                       NMSettingVPN *s_vpn,
+                       NMSettingVpn *s_vpn,
                        const char *secret_name)
 {
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	IodineEditorPrivate *priv = \
+		IODINE_EDITOR_GET_PRIVATE (self);
 
 	NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
 	GtkWidget *widget;
@@ -301,32 +300,30 @@ setup_password_widget (IodinePluginUiWidget *self,
 		value = nm_setting_vpn_get_secret (s_vpn, secret_name);
 		gtk_entry_set_text (GTK_ENTRY (widget), value ? value : "");
 		nm_setting_get_secret_flags (NM_SETTING (s_vpn),
-									 secret_name,
-									 &secret_flags,
-									 NULL);
+		                             secret_name,
+		                             &secret_flags,
+		                             NULL);
 	}
 
 	secret_flags &= ~(NM_SETTING_SECRET_FLAG_NOT_SAVED |
-					  NM_SETTING_SECRET_FLAG_NOT_REQUIRED);
+	                  NM_SETTING_SECRET_FLAG_NOT_REQUIRED);
 	g_object_set_data (G_OBJECT (widget),
-					   "flags",
-					   GUINT_TO_POINTER (secret_flags));
+	                   "flags",
+	                   GUINT_TO_POINTER (secret_flags));
 
 	g_signal_connect (widget, "changed", G_CALLBACK (stuff_changed_cb), self);
 }
 
 static void
-show_toggled_cb (GtkCheckButton *button, IodinePluginUiWidget *self)
+show_toggled_cb (GtkCheckButton *button, IodineEditor *self)
 {
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	IodineEditorPrivate *priv = IODINE_EDITOR_GET_PRIVATE (self);
 	GtkWidget *widget;
 	gboolean visible;
 
 	visible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "password_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "password_entry"));
 	g_assert (widget);
 	gtk_entry_set_visibility (GTK_ENTRY (widget), visible);
 }
@@ -334,13 +331,11 @@ show_toggled_cb (GtkCheckButton *button, IodinePluginUiWidget *self)
 static void
 pw_type_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 {
-	IodinePluginUiWidget *self = IODINE_PLUGIN_UI_WIDGET (user_data);
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	IodineEditor *self = IODINE_EDITOR (user_data);
+	IodineEditorPrivate *priv = IODINE_EDITOR_GET_PRIVATE (self);
 	GtkWidget *entry;
 
-	entry = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												"password_entry"));
+	entry = GTK_WIDGET (gtk_builder_get_object (priv->builder, "password_entry"));
 	g_assert (entry);
 
 	/* If the user chose "Not required", desensitize and clear the correct
@@ -361,14 +356,13 @@ pw_type_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 }
 
 static void
-init_one_pw_combo (IodinePluginUiWidget *self,
-                   NMSettingVPN *s_vpn,
+init_one_pw_combo (IodineEditor *self,
+                   NMSettingVpn *s_vpn,
                    const char *combo_name,
                    const char *secret_key,
                    const char *entry_name)
 {
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	IodineEditorPrivate *priv = IODINE_EDITOR_GET_PRIVATE (self);
 	int active = -1;
 	GtkWidget *widget;
 	GtkListStore *store;
@@ -390,9 +384,9 @@ init_one_pw_combo (IodinePluginUiWidget *self,
 
 	if (s_vpn)
 		nm_setting_get_secret_flags (NM_SETTING (s_vpn),
-									 secret_key,
-									 &pw_flags,
-									 NULL);
+		                             secret_key,
+		                             &pw_flags,
+		                             NULL);
 	if ((active < 0)
 	    && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED)
 	    && !(pw_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
@@ -407,33 +401,30 @@ init_one_pw_combo (IodinePluginUiWidget *self,
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, combo_name));
 	g_assert (widget);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 
-							  active < 0 ? default_idx : active);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (widget),
+	                          active < 0 ? default_idx : active);
 
 	pw_type_combo_changed_cb (widget, self);
 	g_signal_connect (G_OBJECT (widget),
-					  "changed",
-					  G_CALLBACK (pw_type_combo_changed_cb), self);
+	                  "changed",
+	                  G_CALLBACK (pw_type_combo_changed_cb), self);
 }
 
 static gboolean
-init_plugin_ui (IodinePluginUiWidget *self,
-				NMConnection *connection,
-				GError **error)
+init_editor_plugin (IodineEditor *self,
+                NMConnection *connection,
+                GError **error)
 {
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
-	NMSettingVPN *s_vpn;
+	IodineEditorPrivate *priv = IODINE_EDITOR_GET_PRIVATE (self);
+	NMSettingVpn *s_vpn;
 	GtkWidget *widget;
 	const char *value;
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection,
-														NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 
 	priv->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "topdomain_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "topdomain_entry"));
 	if (!widget)
 		return FALSE;
 	gtk_size_group_add_widget (priv->group, widget);
@@ -443,11 +434,10 @@ init_plugin_ui (IodinePluginUiWidget *self,
 			gtk_entry_set_text (GTK_ENTRY (widget), value);
 	}
 	g_signal_connect (G_OBJECT (widget),
-					  "changed",
-					  G_CALLBACK (stuff_changed_cb), self);
+	                  "changed",
+	                  G_CALLBACK (stuff_changed_cb), self);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "nameserver_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "nameserver_entry"));
 	if (!widget)
 		return FALSE;
 	gtk_size_group_add_widget (priv->group, widget);
@@ -457,11 +447,10 @@ init_plugin_ui (IodinePluginUiWidget *self,
 			gtk_entry_set_text (GTK_ENTRY (widget), value);
 	}
 	g_signal_connect (G_OBJECT (widget),
-					  "changed",
-					  G_CALLBACK (stuff_changed_cb), self);
+	                  "changed",
+	                  G_CALLBACK (stuff_changed_cb), self);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "fragsize_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "fragsize_entry"));
 	if (!widget)
 		return FALSE;
 	if (s_vpn) {
@@ -470,19 +459,18 @@ init_plugin_ui (IodinePluginUiWidget *self,
 			gtk_entry_set_text (GTK_ENTRY (widget), value);
 	}
 	g_signal_connect (G_OBJECT (widget),
-					  "changed",
-					  G_CALLBACK (stuff_changed_cb), self);
+	                  "changed",
+	                  G_CALLBACK (stuff_changed_cb), self);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "show_passwords_checkbutton"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "show_passwords_checkbutton"));
 	g_signal_connect (G_OBJECT (widget), "toggled",
 	                  (GCallback) show_toggled_cb,
 	                  self);
 
 	setup_password_widget (self,
-						   "password_entry",
-						   s_vpn,
-						   NM_IODINE_KEY_PASSWORD);
+	                       "password_entry",
+	                       s_vpn,
+	                       NM_IODINE_KEY_PASSWORD);
 
 	init_one_pw_combo (self,
 	                   s_vpn,
@@ -493,19 +481,19 @@ init_plugin_ui (IodinePluginUiWidget *self,
 }
 
 static GObject *
-get_widget (NMVpnPluginUiWidgetInterface *iface)
+get_widget (NMVpnEditor *iface)
 {
-	IodinePluginUiWidget *self = IODINE_PLUGIN_UI_WIDGET (iface);
-	IodinePluginUiWidgetPrivate *priv = IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	IodineEditor *self = IODINE_EDITOR (iface);
+	IodineEditorPrivate *priv = IODINE_EDITOR_GET_PRIVATE (self);
 
 	return G_OBJECT (priv->widget);
 }
 
 static void
-save_password_and_flags (NMSettingVPN *s_vpn,
+save_password_and_flags (NMSettingVpn *s_vpn,
                          GtkBuilder *builder,
                          const char *entry_name,
-						 const char *combo_name,
+                         const char *combo_name,
                          const char *secret_key)
 {
 	NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
@@ -540,14 +528,13 @@ save_password_and_flags (NMSettingVPN *s_vpn,
 }
 
 static gboolean
-update_connection (NMVpnPluginUiWidgetInterface *iface,
+update_connection (NMVpnEditor *iface,
                    NMConnection *connection,
                    GError **error)
 {
-	IodinePluginUiWidget *self = IODINE_PLUGIN_UI_WIDGET (iface);
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
-	NMSettingVPN *s_vpn;
+	IodineEditor *self = IODINE_EDITOR (iface);
+	IodineEditorPrivate *priv = IODINE_EDITOR_GET_PRIVATE (self);
+	NMSettingVpn *s_vpn;
 	GtkWidget *widget;
 	char *str;
 
@@ -556,61 +543,57 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
 	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE,
-				  NM_DBUS_SERVICE_IODINE,
-				  NULL);
+	              NM_DBUS_SERVICE_IODINE,
+	              NULL);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "topdomain_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "topdomain_entry"));
 	g_assert(widget);
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_IODINE_KEY_TOPDOMAIN, str);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "nameserver_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "nameserver_entry"));
 	g_assert(widget);
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_IODINE_KEY_NAMESERVER, str);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-												 "fragsize_entry"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "fragsize_entry"));
 	g_assert(widget);
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
 	if (str && strlen (str))
 		nm_setting_vpn_add_data_item (s_vpn, NM_IODINE_KEY_FRAGSIZE, str);
 
-  	/* User password and flags */
+	/* User password and flags */
 	save_password_and_flags (s_vpn,
-							 priv->builder,
-							 "password_entry",
-							 "pass_type_combo",
-							 NM_IODINE_KEY_PASSWORD);
+	                         priv->builder,
+	                         "password_entry",
+	                         "pass_type_combo",
+	                         NM_IODINE_KEY_PASSWORD);
 
 	nm_connection_add_setting (connection, NM_SETTING (s_vpn));
 	return TRUE;
 }
 
-static NMVpnPluginUiWidgetInterface *
-nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
+static NMVpnEditor *
+nm_vpn_editor_interface_new (NMConnection *connection, GError **error)
 {
-	NMVpnPluginUiWidgetInterface *object;
-	IodinePluginUiWidgetPrivate *priv;
+	NMVpnEditor *object;
+	IodineEditorPrivate *priv;
 	char *ui_file;
 
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
 
-	object = NM_VPN_PLUGIN_UI_WIDGET_INTERFACE  \
-		(g_object_new (IODINE_TYPE_PLUGIN_UI_WIDGET, NULL));
+	object = g_object_new (IODINE_TYPE_EDITOR, NULL);
 
 	if (!object) {
-		g_set_error (error, IODINE_PLUGIN_UI_ERROR, 0,
-					 "could not create iodine object");
+		g_set_error (error, IODINE_EDITOR_PLUGIN_ERROR, 0,
+		             "could not create iodine object");
 		return NULL;
 	}
 
-	priv = IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (object);
+	priv = IODINE_EDITOR_GET_PRIVATE (object);
 	ui_file = g_strdup_printf ("%s/%s", UIDIR, "nm-iodine-dialog.ui");
 	priv->builder = gtk_builder_new ();
 
@@ -619,7 +602,7 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 		g_warning ("Couldn't load builder file: %s",
 		           error && *error ? (*error)->message : "(unknown)");
 		g_clear_error (error);
-		g_set_error (error, IODINE_PLUGIN_UI_ERROR, 0,
+		g_set_error (error, IODINE_EDITOR_PLUGIN_ERROR, 0,
 		             "could not load required resources at %s", ui_file);
 		g_free (ui_file);
 		g_object_unref (object);
@@ -627,17 +610,16 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 	}
 	g_free (ui_file);
 
-	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-													   "iodine-vbox"));
+	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "iodine-vbox"));
 	if (!priv->widget) {
-		g_set_error (error, IODINE_PLUGIN_UI_ERROR, 0,
-					 "could not load UI widget");
+		g_set_error (error, IODINE_EDITOR_PLUGIN_ERROR, 0,
+		             "could not load UI widget");
 		g_object_unref (object);
 		return NULL;
 	}
 	g_object_ref_sink (priv->widget);
 
-	if (!init_plugin_ui (IODINE_PLUGIN_UI_WIDGET (object), connection, error)) {
+	if (!init_editor_plugin (IODINE_EDITOR (object), connection, error)) {
 		g_object_unref (object);
 		return NULL;
 	}
@@ -648,9 +630,9 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 static void
 dispose (GObject *object)
 {
-	IodinePluginUiWidget *plugin = IODINE_PLUGIN_UI_WIDGET (object);
-	IodinePluginUiWidgetPrivate *priv = \
-		IODINE_PLUGIN_UI_WIDGET_GET_PRIVATE (plugin);
+	IodineEditor *plugin = IODINE_EDITOR (object);
+	IodineEditorPrivate *priv = \
+		IODINE_EDITOR_GET_PRIVATE (plugin);
 
 	if (priv->group)
 		g_object_unref (priv->group);
@@ -661,26 +643,26 @@ dispose (GObject *object)
 	if (priv->builder)
 		g_object_unref (priv->builder);
 
-	G_OBJECT_CLASS (iodine_plugin_ui_widget_parent_class)->dispose (object);
+	G_OBJECT_CLASS (iodine_editor_parent_class)->dispose (object);
 }
 
 static void
-iodine_plugin_ui_widget_class_init (IodinePluginUiWidgetClass *req_class)
+iodine_editor_class_init (IodineEditorClass *req_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (req_class);
 
-	g_type_class_add_private (req_class, sizeof (IodinePluginUiWidgetPrivate));
+	g_type_class_add_private (req_class, sizeof (IodineEditorPrivate));
 
 	object_class->dispose = dispose;
 }
 
 static void
-iodine_plugin_ui_widget_init (IodinePluginUiWidget *plugin)
+iodine_editor_init (IodineEditor *plugin)
 {
 }
 
 static void
-iodine_plugin_ui_widget_interface_init (NMVpnPluginUiWidgetInterface *iface_class)
+iodine_editor_interface_init (NMVpnEditorInterface *iface_class)
 {
 	/* interface implementation */
 	iface_class->get_widget = get_widget;
@@ -688,32 +670,32 @@ iodine_plugin_ui_widget_interface_init (NMVpnPluginUiWidgetInterface *iface_clas
 }
 
 static guint32
-get_capabilities (NMVpnPluginUiInterface *iface)
+get_capabilities (NMVpnEditorPlugin *iface)
 {
-	return (NM_VPN_PLUGIN_UI_CAPABILITY_IMPORT |
-			NM_VPN_PLUGIN_UI_CAPABILITY_EXPORT);
+	return (NM_VPN_EDITOR_PLUGIN_CAPABILITY_IMPORT |
+			NM_VPN_EDITOR_PLUGIN_CAPABILITY_EXPORT);
 }
 
-static NMVpnPluginUiWidgetInterface *
-ui_factory (NMVpnPluginUiInterface *iface,
-			NMConnection *connection,
-			GError **error)
+static NMVpnEditor *
+get_editor (NMVpnEditorPlugin *iface,
+            NMConnection *connection,
+            GError **error)
 {
-	return nm_vpn_plugin_ui_widget_interface_new (connection, error);
+	return nm_vpn_editor_interface_new (connection, error);
 }
 
 static void
 get_property (GObject *object, guint prop_id,
-			  GValue *value, GParamSpec *pspec)
+              GValue *value, GParamSpec *pspec)
 {
 	switch (prop_id) {
-	case NM_VPN_PLUGIN_UI_INTERFACE_PROP_NAME:
+	case PROP_NAME:
 		g_value_set_string (value, IODINE_PLUGIN_NAME);
 		break;
-	case NM_VPN_PLUGIN_UI_INTERFACE_PROP_DESC:
+	case PROP_DESC:
 		g_value_set_string (value, IODINE_PLUGIN_DESC);
 		break;
-	case NM_VPN_PLUGIN_UI_INTERFACE_PROP_SERVICE:
+	case PROP_SERVICE:
 		g_value_set_string (value, IODINE_PLUGIN_SERVICE);
 		break;
 	default:
@@ -723,46 +705,45 @@ get_property (GObject *object, guint prop_id,
 }
 
 static void
-iodine_plugin_ui_class_init (IodinePluginUiClass *req_class)
+iodine_editor_plugin_class_init (IodineEditorPluginClass *req_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (req_class);
 
 	object_class->get_property = get_property;
 
 	g_object_class_override_property (object_class,
-									  NM_VPN_PLUGIN_UI_INTERFACE_PROP_NAME,
-									  NM_VPN_PLUGIN_UI_INTERFACE_NAME);
+	                                  PROP_NAME,
+	                                  NM_VPN_EDITOR_PLUGIN_NAME);
 
 	g_object_class_override_property (object_class,
-									  NM_VPN_PLUGIN_UI_INTERFACE_PROP_DESC,
-									  NM_VPN_PLUGIN_UI_INTERFACE_DESC);
+	                                  PROP_DESC,
+	                                  NM_VPN_EDITOR_PLUGIN_DESCRIPTION);
 
 	g_object_class_override_property (object_class,
-									  NM_VPN_PLUGIN_UI_INTERFACE_PROP_SERVICE,
-									  NM_VPN_PLUGIN_UI_INTERFACE_SERVICE);
+	                                  PROP_SERVICE,
+	                                  NM_VPN_EDITOR_PLUGIN_SERVICE);
 }
 
 static void
-iodine_plugin_ui_init (IodinePluginUi *plugin)
+iodine_editor_plugin_init (IodineEditorPlugin *plugin)
 {
 }
 
 static void
-iodine_plugin_ui_interface_init (NMVpnPluginUiInterface *iface_class)
+iodine_editor_plugin_interface_init (NMVpnEditorPluginInterface *iface_class)
 {
 	/* interface implementation */
-	iface_class->ui_factory = ui_factory;
+	iface_class->get_editor = get_editor;
 	iface_class->get_capabilities = get_capabilities;
 	iface_class->import_from_file = import;
 	iface_class->export_to_file = export;
 }
 
-G_MODULE_EXPORT NMVpnPluginUiInterface *
-nm_vpn_plugin_ui_factory (GError **error)
+G_MODULE_EXPORT NMVpnEditorPlugin *
+nm_vpn_editor_plugin_factory (GError **error)
 {
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
 
-	return NM_VPN_PLUGIN_UI_INTERFACE (g_object_new (IODINE_TYPE_PLUGIN_UI,
-													 NULL));
+	return g_object_new (IODINE_TYPE_EDITOR_PLUGIN, NULL);
 }
